package com.gic.cloud.web.controller;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.cloud.web.auth.DataAuthUtils;
import com.gic.cloud.web.constant.StoreChannelEnum;
import com.gic.cloud.web.utils.LevelUtil;
import com.gic.cloud.web.vo.BaseSalesVo;
import com.gic.cloud.web.vo.LevelVo;
import com.gic.commons.util.CollectionUtil;
import com.gic.data.api.dto.BaseDTO;
import com.gic.data.api.dto.BaseSalesDTO;
import com.gic.data.api.qo.common.CommonQO;
import com.gic.enterprise.dto.WmStoreDTO;
import com.gic.enterprise.service.WmStoreApiService;
import com.gic.mall.share.api.dto.shop.ShopDTO;
import com.gic.mall.share.api.enums.ShopTypeEnum;
import com.gic.mall.share.api.service.ShopApiService;
import com.gic.store.dto.ClerkDTO;
import com.gic.store.service.ClerkApiService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Component
public class GoDownComponent {
    private Logger log = LogManager.getLogger(GoDownComponent.class);
    @Autowired
    private ShopApiService shopApiService;
    @Autowired
    private WmStoreApiService wmStoreApiService;
    @Autowired
    private StoreSearchUtils storeSearchUtils;
    @Autowired
    private DataAuthUtils dataAuthUtils;
    @Autowired
    private ClerkApiService clerkApiService;

    public void parseTotal(ServiceResponse<?> response, BaseSalesVo vo){
        if(response.isSuccess() && response.getResult() != null){
            Page<BaseDTO> page = (Page<BaseDTO>) response.getResult();
            if(CollectionUtils.isEmpty(page.getResult())){
                return;
            }
            BaseDTO total = page.getResult().get(0);
            if(total != null){
                total.setAreaId("0");
                total.setAreaName("合计/均值");
                total.setId("0");
                total.setName("合计/均值");
                vo.setTotal(total);
            }
        }
    }

    public void parseWgs(ServiceResponse<?> response, BaseSalesVo vo){
        if(response.isSuccess() && response.getResult() != null ){
            Page<BaseDTO> page = (Page<BaseDTO>) response.getResult();
            if(CollectionUtils.isEmpty(page.getResult())){
                return;
            }
            BaseDTO wgs = page.getResult().get(0);
            if(wgs != null){
                wgs.setAreaName("无归属");
                wgs.setAreaId("-1");
                wgs.setName("无归属");
                wgs.setId("-1");
                vo.setWgs(wgs);
            }
        }
    }

    public LevelVo setValue(CommonQO params, BaseSalesVo vo){
        params.setOnlineStoreIdList(storeSearchUtils.onLineStoreSearch(params.getUserId(), params.getEnterpriseId(), params.getSearchJSON()));
        params.setStoreInfoIdList(storeSearchUtils.storeSearch(params.getUserId(), params.getEnterpriseId(), params.getSearchJSON()));
        params.setMemberCardIdList(dataAuthUtils.getMbrAreaId(params.getUserId(), params.getEnterpriseId()));
        /******线下渠道需要点击下钻，该值只会在线下时才会使用********/
        LevelVo levelVo = this.storeSearchUtils.getLevel(params.getUserId(), params.getEnterpriseId(), params.getSearchJSON());
        if(params.getLevel() == null){
            params.setLevel(levelVo.getLevel());
        }
        if(params.getLevel().intValue() > levelVo.getMaxLevel().intValue()){
            if(params.getLevel().intValue()- levelVo.getMaxLevel().intValue() == 1){
                params.setLevel(7);
            }
        }
        List<Integer> childrenStoreGroupIdList = this.storeSearchUtils.getChildrenStoreGroupIdList(params.getEnterpriseId(), params.getParentId(), params.getLevel());
        params.setGroupIdList(childrenStoreGroupIdList);
        /****首页进入基础页面时，如果渠道为空，根据当前条件判断渠道，channel==null表示多渠道***/
        if(params.getChannel() == null){
            /**********设置层级名称，只有在首次进入页面会展示，下钻均显示返回上一级*********/
            Integer newChannel = this.storeSearchUtils.getChannel(params.getUserId(), params.getEnterpriseId(), params.getSearchJSON());
            params.setChannel(newChannel);
        }
        /***********如果是线下的第一层级，需要展示切换层级按钮*************/
        if(params.getChannel() == StoreChannelEnum.OFFLINE.getChannel() && params.getLevel() == levelVo.getLevel()
                || (params.getChannel() != null && (params.getChannel() == 7 || params.getChannel() == 8))){
            vo.setChangeLevel(1);
        }
        if(params.getChannel() == null){
            vo.setLevelName("所有渠道");
        } else if(params.getChannel() == StoreChannelEnum.OFFLINE.getChannel()){
            vo.setLevelName(LevelUtil.getLevelName(params.getLevel()));
        } else {
            vo.setLevelName("店铺层级");
        }
        return levelVo;
    }


    public void setValueTotal(CommonQO params, LevelVo levelVo){
        Integer wgs = storeSearchUtils.isWgs(params.getUserId(), params.getEnterpriseId(), params.getSearchJSON());
        /**所有渠道或者线下渠道第一层级 合计算上无归属数据**/
        if((params.getChannel() == null || (params.getChannel() == StoreChannelEnum.OFFLINE.getChannel() && params.getLevel() == levelVo.getLevel())) && wgs == 1){
            if (CollectionUtils.isEmpty(params.getStoreInfoIdList())) {
                params.setStoreInfoIdList(new ArrayList<>());
            }
            if (CollectionUtils.isEmpty(params.getOnlineStoreIdList())) {
                params.setOnlineStoreIdList(new ArrayList<>());
            }
            params.getStoreInfoIdList().add(-1);
            params.getOnlineStoreIdList().add("-1");
        }
    }

    public void setValueTotal(CommonQO params){
        Integer wgs = storeSearchUtils.isWgs(params.getUserId(), params.getEnterpriseId(), params.getSearchJSON());
        /**所有渠道或者线下渠道第一层级 合计算上无归属数据**/
        if(wgs == 1){
            if (CollectionUtils.isEmpty(params.getStoreInfoIdList())) {
                params.setStoreInfoIdList(new ArrayList<>());
            }
            if (CollectionUtils.isEmpty(params.getOnlineStoreIdList())) {
                params.setOnlineStoreIdList(new ArrayList<>());
            }
            params.getStoreInfoIdList().add(-1);
            params.getOnlineStoreIdList().add("-1");
        }
    }

    public void setValueWgs(CommonQO params, LevelVo levelVo){
        Integer wgs = storeSearchUtils.isWgs(params.getUserId(), params.getEnterpriseId(), params.getSearchJSON());
        if(params.getChannel() != null && params.getChannel() == 1 && params.getLevel() == levelVo.getLevel() && wgs == 1){
            params.setChannel(null);
            if (CollectionUtils.isEmpty(params.getStoreInfoIdList())) {
                params.setStoreInfoIdList(new ArrayList<>());
            }
            if (CollectionUtils.isEmpty(params.getOnlineStoreIdList())) {
                params.setOnlineStoreIdList(new ArrayList<>());
            }
            params.getStoreInfoIdList().clear();
            params.getOnlineStoreIdList().clear();
            params.getStoreInfoIdList().add(-1);
            params.getOnlineStoreIdList().add("-1");
        }
    }

    public void setValueWgs(CommonQO params){
        Integer wgs = storeSearchUtils.isWgs(params.getUserId(), params.getEnterpriseId(), params.getSearchJSON());
        if(wgs == 1){
            params.setChannel(null);
            if (CollectionUtils.isEmpty(params.getStoreInfoIdList())) {
                params.setStoreInfoIdList(new ArrayList<>());
            }
            if (CollectionUtils.isEmpty(params.getOnlineStoreIdList())) {
                params.setOnlineStoreIdList(new ArrayList<>());
            }
            params.getStoreInfoIdList().clear();
            params.getOnlineStoreIdList().clear();
            params.getStoreInfoIdList().add(-1);
            params.getOnlineStoreIdList().add("-1");
        }
    }

    public Page<?> parseData(Page<?> page , Integer channel, Integer enterpriseId, Integer level, Integer maxLevel){
        if(page != null && CollectionUtils.isNotEmpty(page.getResult())){
            if(channel == null){  //全渠道
                page.getResult().stream().forEach(b -> {
                    BaseDTO baseSalesDTO = (BaseDTO) b;
                    baseSalesDTO.setLevel(level - 1);
                    if(StoreChannelEnum.OFFLINE.getChannel() == (baseSalesDTO instanceof BaseSalesDTO ? Integer.valueOf(baseSalesDTO.getId()) : Integer.valueOf(baseSalesDTO.getAreaId()))){
                        baseSalesDTO.setAreaName(StoreChannelEnum.OFFLINE.getMessage());
                        baseSalesDTO.setName(StoreChannelEnum.OFFLINE.getMessage());
                        baseSalesDTO.setChannel(StoreChannelEnum.OFFLINE.getChannel());
                        baseSalesDTO.setHasChildren(1);
                    }
                    if(StoreChannelEnum.GICMALL.getChannel() == (baseSalesDTO instanceof BaseSalesDTO ? Integer.valueOf(baseSalesDTO.getId()) : Integer.valueOf(baseSalesDTO.getAreaId()))){
                        baseSalesDTO.setName(StoreChannelEnum.GICMALL.getMessage());
                        baseSalesDTO.setAreaName(StoreChannelEnum.GICMALL.getMessage());
                        baseSalesDTO.setChannel(StoreChannelEnum.GICMALL.getChannel());
                        baseSalesDTO.setHasChildren(1);
                    }
                    if(StoreChannelEnum.WMMALL.getChannel() == (baseSalesDTO instanceof BaseSalesDTO ? Integer.valueOf(baseSalesDTO.getId()) : Integer.valueOf(baseSalesDTO.getAreaId()))){
                        baseSalesDTO.setName(StoreChannelEnum.WMMALL.getMessage());
                        baseSalesDTO.setAreaName(StoreChannelEnum.WMMALL.getMessage());
                        baseSalesDTO.setChannel(StoreChannelEnum.WMMALL.getChannel());
                        baseSalesDTO.setHasChildren(1);
                    }
                    if(StoreChannelEnum.TIANMAO.getChannel() == (baseSalesDTO instanceof BaseSalesDTO ? Integer.valueOf(baseSalesDTO.getId()) : Integer.valueOf(baseSalesDTO.getAreaId()))){
                        baseSalesDTO.setName(StoreChannelEnum.TIANMAO.getMessage());
                        baseSalesDTO.setAreaName(StoreChannelEnum.TIANMAO.getMessage());
                        baseSalesDTO.setChannel(StoreChannelEnum.TIANMAO.getChannel());
                        baseSalesDTO.setHasChildren(1);
                    }
                });
            }
            if(channel == StoreChannelEnum.GICMALL.getChannel()){
                List<ShopDTO> result = shopApiService.getAllShopByEnterpriseIdAndName(enterpriseId, null, ShopTypeEnum.MALL_SHOP.getCode()).getResult();
                if(CollectionUtils.isNotEmpty(result)){
                    page.getResult().stream().forEach(b -> {
                        BaseDTO baseSalesDTO = (BaseDTO) b;
                        baseSalesDTO.setLevel(level - 1);
                        for(ShopDTO shopDTO : result){
                            if(baseSalesDTO instanceof BaseSalesDTO){
                                if(baseSalesDTO.getId().equals(shopDTO.getId()+"")){
                                    baseSalesDTO.setName(shopDTO.getName());
                                    baseSalesDTO.setChannel(channel);
                                }
                            }else {
                                if(baseSalesDTO.getAreaId().equals(shopDTO.getId()+"")){
                                    baseSalesDTO.setAreaName(shopDTO.getName());
                                    baseSalesDTO.setChannel(channel);
                                }
                            }
                        }
                    });
                }
            }
            if(channel == StoreChannelEnum.WMMALL.getChannel()){
                List<WmStoreDTO> result1 = wmStoreApiService.listWmStore(enterpriseId, null).getResult();
                if(CollectionUtils.isNotEmpty(result1)){
                    page.getResult().stream().forEach(b -> {
                        BaseDTO baseSalesDTO = (BaseDTO) b;
                        baseSalesDTO.setLevel(level - 1);
                        for(WmStoreDTO dto : result1){
                            if(baseSalesDTO instanceof BaseSalesDTO){
                                if(baseSalesDTO.getId().equals(dto.getWmMallStoreId()+"")){
                                    baseSalesDTO.setName(dto.getWmMainAccount());
                                    baseSalesDTO.setChannel(channel);
                                }
                            }else {
                                if(baseSalesDTO.getAreaId().equals(dto.getWmMallStoreId()+"")){
                                    baseSalesDTO.setAreaName(dto.getWmMainAccount());
                                    baseSalesDTO.setChannel(channel);
                                }
                            }
                        }
                    });
                }
            }
            if(channel == StoreChannelEnum.OFFLINE.getChannel()){
                List<ClerkDTO> clerkDTOS = new ArrayList<>();
                if(level.intValue() == 8){
                    List<String> clerkIdList = page.getResult().stream().map(b -> {
                        BaseDTO baseSalesDTO = (BaseDTO) b;
                        return baseSalesDTO.getAreaId() == null ? baseSalesDTO.getId() : baseSalesDTO.getAreaId();
                    }).collect(Collectors.toList());
                    log.info("clerkIdList:{}", JSON.toJSONString(clerkIdList));
                    clerkDTOS = this.clerkApiService.listClerkByIds(enterpriseId, StringUtils.join(clerkIdList, ",")).getResult();
                    log.info("clerkDtos:{}", clerkDTOS);
                }
                final Map<Integer, ClerkDTO> clerkDTOMap = CollectionUtil.toMap(clerkDTOS, "clerkId");
                log.info("clerkDTOMap:{}", JSON.toJSONString(clerkDTOMap));
                page.getResult().stream().forEach(b -> {
                    BaseDTO baseSalesDTO = (BaseDTO) b;
                    baseSalesDTO.setLevel(level);
                    baseSalesDTO.setChannel(StoreChannelEnum.OFFLINE.getChannel());
                    baseSalesDTO.setHasChildren(level <= maxLevel ? 1 : 0);
                    if(clerkDTOMap != null && level.intValue() == 8){
                        ClerkDTO clerkDTO = clerkDTOMap.get(baseSalesDTO.getAreaId() == null ? Integer.valueOf(baseSalesDTO.getId()) : Integer.valueOf(baseSalesDTO.getAreaId()));
                        if(clerkDTO != null){
                            baseSalesDTO.setName(clerkDTO.getClerkName());
                        }else{
                            baseSalesDTO.setName("未知");
                        }
                    }
                });
            }
        }
        return page;
    }
}
